"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const html_manager_1 = require("@jupyter-widgets/html-manager");
const base_1 = require("@jupyter-widgets/base");
const pWidget = require("@phosphor/widgets");
const utils = require("@jupyter-widgets/base");
const kernel_1 = require("./kernel");
class WidgetManager extends html_manager_1.HTMLManager {
    constructor(options) {
        super(options);
        this.kernel = new kernel_1.DefaultKernel();
        this.kernel.registerCommTarget(this.comm_target_name, (comm, msg) => {
            this.handle_comm_open(new base_1.shims.services.Comm(comm), msg);
        });
    }
    async render(bundle, el) {
        const { spec, state } = bundle;
        this._last_state = state;
        try {
            const models = await this.set_state(state);
            const model = models.find((item) => item.model_id == spec.model_id);
            if (model != null) {
                await this.display_model(undefined, model, { el });
            }
        }
        finally {
            this._last_state = null;
        }
    }
    async _create_comm(target_name, model_id, data, metadata, buffers) {
        const comm = this.kernel.connectToComm(target_name, model_id);
        if (data || metadata) {
            comm.open(data, metadata, buffers);
        }
        return new base_1.shims.services.Comm(comm);
    }
    _get_comm_info() {
        return Promise.resolve(this._last_state.state);
    }
    display_view(_msg, view, options) {
        return Promise.resolve(view).then(view => {
            pWidget.Widget.attach(view.pWidget, options.el);
            view.on("remove", () => { console.log("view removed", view); });
            return view;
        });
    }
    loadClass(className, moduleName, moduleVersion) {
        return super.loadClass(className, moduleName, moduleVersion);
    }
    set_state(state) {
        this._last_state = state;
        // Check to make sure that it"s the same version we are parsing.
        if (!(state.version_major && state.version_major <= 2)) {
            throw new Error("Unsupported widget state format");
        }
        const models = state.state;
        // Recreate all the widget models for the given widget manager state.
        const all_models = this._get_comm_info().then((live_comms) => {
            return Promise.all(Object.keys(models).map((model_id) => {
                // First put back the binary buffers
                const decode = { base64: utils.base64ToBuffer, hex: utils.hexToBuffer };
                const model = models[model_id];
                const model_state = model.state;
                if (model.buffers) {
                    const buffer_paths = model.buffers.map((b) => b.path);
                    // put_buffers expects buffers to be DataViews
                    const buffers = model.buffers.map((b) => new DataView(decode[b.encoding](b.data)));
                    utils.put_buffers(model.state, buffer_paths, buffers);
                }
                // If the model has already been created, set its state and then
                // return it.
                const promise = this.get_model(model_id);
                if (promise != null) {
                    return promise.then((model) => {
                        // deserialize state
                        return model.constructor._deserialize_state(model_state || {}, this).then((attributes) => {
                            model.set_state(attributes);
                            return model;
                        });
                    });
                }
                const model_create = {
                    model_id: model_id,
                    model_name: model.model_name,
                    model_module: model.model_module,
                    model_module_version: model.model_module_version
                };
                if (live_comms.hasOwnProperty(model_id)) {
                    // This connects to an existing comm if it exists, and
                    // should *not* send a comm open message.
                    return this._create_comm(this.comm_target_name, model_id).then((comm) => {
                        return this.new_model(Object.assign(Object.assign({}, model_create), { comm }), model_state);
                    });
                }
                else {
                    return this.new_model(model_create, model_state);
                }
            }));
        });
        return all_models;
    }
}
exports.WidgetManager = WidgetManager;
